@@ -141,13 +141,14 @@ assert!(!re.is_match("abc").unwrap());
 [regex]: https://crates.io/crates/regex
 */
 
-#![doc(html_root_url = "https://docs.rs/fancy-regex/0.4.0")]
+#![doc(html_root_url = "https://docs.rs/fancy-regex/0.7.1")]
 #![deny(missing_docs)]
 #![deny(missing_debug_implementations)]
 
 use std::fmt;
 use std::fmt::{Debug, Formatter};
-use std::ops::Range;
+use std::ops::{Index, Range};
+use std::str::FromStr;
 use std::sync::Arc;
 use std::usize;
 
@@ -156,6 +157,7 @@ mod compile;
 mod error;
 mod expand;
 mod parse;
+mod replacer;
 mod vm;
 
 use crate::analyze::analyze;
@@ -165,6 +167,8 @@ use crate::vm::Prog;
 
 pub use crate::error::{Error, Result};
 pub use crate::expand::Expander;
+pub use crate::replacer::{NoExpand, Replacer, ReplacerRef};
+use std::borrow::Cow;
 
 const MAX_RECURSION: usize = 64;
 
@@ -175,12 +179,14 @@ const MAX_RECURSION: usize = 64;
 pub struct RegexBuilder(RegexOptions);
 
 /// A compiled regular expression.
+#[derive(Clone)]
 pub struct Regex {
     inner: RegexImpl,
     named_groups: Arc<NamedGroups>,
 }
 
 // Separate enum because we don't want to expose any of this
+#[derive(Clone)]
 enum RegexImpl {
     // Do we want to box this? It's pretty big...
     Wrap {
@@ -202,6 +208,125 @@ pub struct Match<'t> {
     end: usize,
 }
 
+/// An iterator over all non-overlapping matches for a particular string.
+///
+/// The iterator yields a `Result<Match>`. The iterator stops when no more
+/// matches can be found.
+///
+/// `'r` is the lifetime of the compiled regular expression and `'t` is the
+/// lifetime of the matched string.
+#[derive(Debug)]
+pub struct Matches<'r, 't> {
+    re: &'r Regex,
+    text: &'t str,
+    last_end: usize,
+    last_match: Option<usize>,
+}
+
+impl<'r, 't> Matches<'r, 't> {
+    /// Return the text being searched.
+    pub fn text(&self) -> &'t str {
+        self.text
+    }
+
+    /// Return the underlying regex.
+    pub fn regex(&self) -> &'r Regex {
+        &self.re
+    }
+}
+
+impl<'r, 't> Iterator for Matches<'r, 't> {
+    type Item = Result<Match<'t>>;
+
+    /// Adapted from the `regex` crate. Calls `find_from_pos` repeatedly.
+    /// Ignores empty matches immediately after a match.
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.last_end > self.text.len() {
+            return None;
+        }
+
+        let mat = match self.re.find_from_pos(self.text, self.last_end) {
+            Err(error) => return Some(Err(error)),
+            Ok(None) => return None,
+            Ok(Some(mat)) => mat,
+        };
+
+        if mat.start == mat.end {
+            // This is an empty match. To ensure we make progress, start
+            // the next search at the smallest possible starting position
+            // of the next match following this one.
+            self.last_end = next_utf8(self.text, mat.end);
+            // Don't accept empty matches immediately following a match.
+            // Just move on to the next match.
+            if Some(mat.end) == self.last_match {
+                return self.next();
+            }
+        } else {
+            self.last_end = mat.end;
+        }
+
+        self.last_match = Some(mat.end);
+
+        Some(Ok(mat))
+    }
+}
+
+/// An iterator that yields all non-overlapping capture groups matching a
+/// particular regular expression.
+///
+/// The iterator stops when no more matches can be found.
+///
+/// `'r` is the lifetime of the compiled regular expression and `'t` is the
+/// lifetime of the matched string.
+#[derive(Debug)]
+pub struct CaptureMatches<'r, 't>(Matches<'r, 't>);
+
+impl<'r, 't> CaptureMatches<'r, 't> {
+    /// Return the text being searched.
+    pub fn text(&self) -> &'t str {
+        self.0.text
+    }
+
+    /// Return the underlying regex.
+    pub fn regex(&self) -> &'r Regex {
+        &self.0.re
+    }
+}
+
+impl<'r, 't> Iterator for CaptureMatches<'r, 't> {
+    type Item = Result<Captures<'t>>;
+
+    /// Adapted from the `regex` crate. Calls `captures_from_pos` repeatedly.
+    /// Ignores empty matches immediately after a match.
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.0.last_end > self.0.text.len() {
+            return None;
+        }
+
+        let captures = match self.0.re.captures_from_pos(self.0.text, self.0.last_end) {
+            Err(error) => return Some(Err(error)),
+            Ok(None) => return None,
+            Ok(Some(captures)) => captures,
+        };
+
+        let mat = captures
+            .get(0)
+            .expect("`Captures` is expected to have entire match at 0th position");
+        if mat.start == mat.end {
+            self.0.last_end = next_utf8(self.0.text, mat.end);
+            if Some(mat.end) == self.0.last_match {
+                return self.next();
+            }
+        } else {
+            self.0.last_end = mat.end;
+        }
+
+        self.0.last_match = Some(mat.end);
+
+        Some(Ok(captures))
+    }
+}
+
 /// A set of capture groups found for a regex.
 #[derive(Debug)]
 pub struct Captures<'t> {
@@ -304,6 +429,22 @@ impl fmt::Debug for Regex {
     }
 }
 
+impl fmt::Display for Regex {
+    /// Shows the original regular expression
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.as_str())
+    }
+}
+
+impl FromStr for Regex {
+    type Err = Error;
+
+    /// Attempts to parse a string into a regular expression
+    fn from_str(s: &str) -> Result<Regex> {
+        Regex::new(s)
+    }
+}
+
 impl Regex {
     /// Parse and compile a regex with default options, see `RegexBuilder`.
     ///
@@ -401,9 +542,37 @@ impl Regex {
         }
     }
 
+    /// Returns an iterator for each successive non-overlapping match in `text`.
+    ///
+    /// If you have capturing groups in your regex that you want to extract, use the [Regex::captures_iter()]
+    /// method.
+    ///
+    /// # Example
+    ///
+    /// Find all words followed by an exclamation point:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    ///
+    /// let re = Regex::new(r"\w+(?=!)").unwrap();
+    /// let mut matches = re.find_iter("so fancy! even with! iterators!");
+    /// assert_eq!(matches.next().unwrap().unwrap().as_str(), "fancy");
+    /// assert_eq!(matches.next().unwrap().unwrap().as_str(), "with");
+    /// assert_eq!(matches.next().unwrap().unwrap().as_str(), "iterators");
+    /// assert!(matches.next().is_none());
+    /// ```
+    pub fn find_iter<'r, 't>(&'r self, text: &'t str) -> Matches<'r, 't> {
+        Matches {
+            re: &self,
+            text,
+            last_end: 0,
+            last_match: None,
+        }
+    }
+
     /// Find the first match in the input text.
     ///
-    /// If you have capturing groups in your regex that you want to extract, use the [captures()]
+    /// If you have capturing groups in your regex that you want to extract, use the [Regex::captures()]
     /// method.
     ///
     /// # Example
@@ -417,17 +586,68 @@ impl Regex {
     /// assert_eq!(re.find("so fancy!").unwrap().unwrap().as_str(), "fancy");
     /// ```
     pub fn find<'t>(&self, text: &'t str) -> Result<Option<Match<'t>>> {
+        self.find_from_pos(text, 0)
+    }
+
+    /// Returns the first match in `text`, starting from the specified byte position `pos`.
+    ///
+    /// # Examples
+    ///
+    /// Finding match starting at a position:
+    ///
+    /// ```
+    /// # use fancy_regex::Regex;
+    /// let re = Regex::new(r"(?m:^)(\d+)").unwrap();
+    /// let text = "1 test 123\n2 foo";
+    /// let mat = re.find_from_pos(text, 7).unwrap().unwrap();
+    ///
+    /// assert_eq!(mat.start(), 11);
+    /// assert_eq!(mat.end(), 12);
+    /// ```
+    ///
+    /// Note that in some cases this is not the same as using the `find`
+    /// method and passing a slice of the string, see [Regex::captures_from_pos()] for details.
+    pub fn find_from_pos<'t>(&self, text: &'t str, pos: usize) -> Result<Option<Match<'t>>> {
         match &self.inner {
             RegexImpl::Wrap { inner, .. } => Ok(inner
-                .find(text)
+                .find_at(text, pos)
                 .map(|m| Match::new(text, m.start(), m.end()))),
             RegexImpl::Fancy { prog, options, .. } => {
-                let result = vm::run(prog, text, 0, 0, options)?;
+                let result = vm::run(prog, text, pos, 0, options)?;
                 Ok(result.map(|saves| Match::new(text, saves[0], saves[1])))
             }
         }
     }
 
+    /// Returns an iterator over all the non-overlapping capture groups matched in `text`.
+    ///
+    /// # Examples
+    ///
+    /// Finding all matches and capturing parts of each:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    ///
+    /// let re = Regex::new(r"(\d{4})-(\d{2})").unwrap();
+    /// let text = "It was between 2018-04 and 2020-01";
+    /// let mut all_captures = re.captures_iter(text);
+    ///
+    /// let first = all_captures.next().unwrap().unwrap();
+    /// assert_eq!(first.get(1).unwrap().as_str(), "2018");
+    /// assert_eq!(first.get(2).unwrap().as_str(), "04");
+    /// assert_eq!(first.get(0).unwrap().as_str(), "2018-04");
+    ///
+    /// let second = all_captures.next().unwrap().unwrap();
+    /// assert_eq!(second.get(1).unwrap().as_str(), "2020");
+    /// assert_eq!(second.get(2).unwrap().as_str(), "01");
+    /// assert_eq!(second.get(0).unwrap().as_str(), "2020-01");
+    ///
+    /// assert!(all_captures.next().is_none());
+    /// ```
+    pub fn captures_iter<'r, 't>(&'r self, text: &'t str) -> CaptureMatches<'r, 't> {
+        CaptureMatches(self.find_iter(text))
+    }
+
     /// Returns the capture groups for the first match in `text`.
     ///
     /// If no match is found, then `Ok(None)` is returned.
@@ -472,7 +692,7 @@ impl Regex {
     /// ```
     ///
     /// Note that in some cases this is not the same as using the `captures`
-    /// methods and passing a slice of the string, see the capture that we get
+    /// method and passing a slice of the string, see the capture that we get
     /// when we do this:
     ///
     /// ```
@@ -541,6 +761,178 @@ impl Regex {
             RegexImpl::Fancy { prog, .. } => prog.debug_print(),
         }
     }
+
+    /// Replaces the leftmost-first match with the replacement provided.
+    /// The replacement can be a regular string (where `$N` and `$name` are
+    /// expanded to match capture groups) or a function that takes the matches'
+    /// `Captures` and returns the replaced string.
+    ///
+    /// If no match is found, then a copy of the string is returned unchanged.
+    ///
+    /// # Replacement string syntax
+    ///
+    /// All instances of `$name` in the replacement text is replaced with the
+    /// corresponding capture group `name`.
+    ///
+    /// `name` may be an integer corresponding to the index of the
+    /// capture group (counted by order of opening parenthesis where `0` is the
+    /// entire match) or it can be a name (consisting of letters, digits or
+    /// underscores) corresponding to a named capture group.
+    ///
+    /// If `name` isn't a valid capture group (whether the name doesn't exist
+    /// or isn't a valid index), then it is replaced with the empty string.
+    ///
+    /// The longest possible name is used. e.g., `$1a` looks up the capture
+    /// group named `1a` and not the capture group at index `1`. To exert more
+    /// precise control over the name, use braces, e.g., `${1}a`.
+    ///
+    /// To write a literal `$` use `$$`.
+    ///
+    /// # Examples
+    ///
+    /// Note that this function is polymorphic with respect to the replacement.
+    /// In typical usage, this can just be a normal string:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    /// let re = Regex::new("[^01]+").unwrap();
+    /// assert_eq!(re.replace("1078910", ""), "1010");
+    /// ```
+    ///
+    /// But anything satisfying the `Replacer` trait will work. For example,
+    /// a closure of type `|&Captures| -> String` provides direct access to the
+    /// captures corresponding to a match. This allows one to access
+    /// capturing group matches easily:
+    ///
+    /// ```rust
+    /// # use fancy_regex::{Regex, Captures};
+    /// let re = Regex::new(r"([^,\s]+),\s+(\S+)").unwrap();
+    /// let result = re.replace("Springsteen, Bruce", |caps: &Captures| {
+    ///     format!("{} {}", &caps[2], &caps[1])
+    /// });
+    /// assert_eq!(result, "Bruce Springsteen");
+    /// ```
+    ///
+    /// But this is a bit cumbersome to use all the time. Instead, a simple
+    /// syntax is supported that expands `$name` into the corresponding capture
+    /// group. Here's the last example, but using this expansion technique
+    /// with named capture groups:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    /// let re = Regex::new(r"(?P<last>[^,\s]+),\s+(?P<first>\S+)").unwrap();
+    /// let result = re.replace("Springsteen, Bruce", "$first $last");
+    /// assert_eq!(result, "Bruce Springsteen");
+    /// ```
+    ///
+    /// Note that using `$2` instead of `$first` or `$1` instead of `$last`
+    /// would produce the same result. To write a literal `$` use `$$`.
+    ///
+    /// Sometimes the replacement string requires use of curly braces to
+    /// delineate a capture group replacement and surrounding literal text.
+    /// For example, if we wanted to join two words together with an
+    /// underscore:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    /// let re = Regex::new(r"(?P<first>\w+)\s+(?P<second>\w+)").unwrap();
+    /// let result = re.replace("deep fried", "${first}_$second");
+    /// assert_eq!(result, "deep_fried");
+    /// ```
+    ///
+    /// Without the curly braces, the capture group name `first_` would be
+    /// used, and since it doesn't exist, it would be replaced with the empty
+    /// string.
+    ///
+    /// Finally, sometimes you just want to replace a literal string with no
+    /// regard for capturing group expansion. This can be done by wrapping a
+    /// byte string with `NoExpand`:
+    ///
+    /// ```rust
+    /// # use fancy_regex::Regex;
+    /// use fancy_regex::NoExpand;
+    ///
+    /// let re = Regex::new(r"(?P<last>[^,\s]+),\s+(\S+)").unwrap();
+    /// let result = re.replace("Springsteen, Bruce", NoExpand("$2 $last"));
+    /// assert_eq!(result, "$2 $last");
+    /// ```
+    pub fn replace<'t, R: Replacer>(&self, text: &'t str, rep: R) -> Cow<'t, str> {
+        self.replacen(text, 1, rep)
+    }
+
+    /// Replaces all non-overlapping matches in `text` with the replacement
+    /// provided. This is the same as calling `replacen` with `limit` set to
+    /// `0`.
+    ///
+    /// See the documentation for `replace` for details on how to access
+    /// capturing group matches in the replacement string.
+    pub fn replace_all<'t, R: Replacer>(&self, text: &'t str, rep: R) -> Cow<'t, str> {
+        self.replacen(text, 0, rep)
+    }
+
+    /// Replaces at most `limit` non-overlapping matches in `text` with the
+    /// replacement provided. If `limit` is 0, then all non-overlapping matches
+    /// are replaced.
+    ///
+    /// See the documentation for `replace` for details on how to access
+    /// capturing group matches in the replacement string.
+    pub fn replacen<'t, R: Replacer>(
+        &self,
+        text: &'t str,
+        limit: usize,
+        mut rep: R,
+    ) -> Cow<'t, str> {
+        // If we know that the replacement doesn't have any capture expansions,
+        // then we can fast path. The fast path can make a tremendous
+        // difference:
+        //
+        //   1) We use `find_iter` instead of `captures_iter`. Not asking for
+        //      captures generally makes the regex engines faster.
+        //   2) We don't need to look up all of the capture groups and do
+        //      replacements inside the replacement string. We just push it
+        //      at each match and be done with it.
+        if let Some(rep) = rep.no_expansion() {
+            let mut it = self.find_iter(text).enumerate().peekable();
+            if it.peek().is_none() {
+                return Cow::Borrowed(text);
+            }
+            let mut new = String::with_capacity(text.len());
+            let mut last_match = 0;
+            for (i, m) in it {
+                let m = m.unwrap();
+                if limit > 0 && i >= limit {
+                    break;
+                }
+                new.push_str(&text[last_match..m.start()]);
+                new.push_str(&rep);
+                last_match = m.end();
+            }
+            new.push_str(&text[last_match..]);
+            return Cow::Owned(new);
+        }
+
+        // The slower path, which we use if the replacement needs access to
+        // capture groups.
+        let mut it = self.captures_iter(text).enumerate().peekable();
+        if it.peek().is_none() {
+            return Cow::Borrowed(text);
+        }
+        let mut new = String::with_capacity(text.len());
+        let mut last_match = 0;
+        for (i, cap) in it {
+            let cap = cap.unwrap();
+            if limit > 0 && i >= limit {
+                break;
+            }
+            // unwrap on 0 is OK because captures only reports matches
+            let m = cap.get(0).unwrap();
+            new.push_str(&text[last_match..m.start()]);
+            rep.replace_append(&cap, &mut new);
+            last_match = m.end();
+        }
+        new.push_str(&text[last_match..]);
+        Cow::Owned(new)
+    }
 }
 
 impl<'t> Match<'t> {
@@ -664,6 +1056,53 @@ impl<'t> Captures<'t> {
     }
 }
 
+/// Copied from [`regex::Captures`]...
+///
+/// Get a group by index.
+///
+/// `'t` is the lifetime of the matched text.
+///
+/// The text can't outlive the `Captures` object if this method is
+/// used, because of how `Index` is defined (normally `a[i]` is part
+/// of `a` and can't outlive it); to do that, use `get()` instead.
+///
+/// # Panics
+///
+/// If there is no group at the given index.
+impl<'t> Index<usize> for Captures<'t> {
+    type Output = str;
+
+    fn index(&self, i: usize) -> &str {
+        self.get(i)
+            .map(|m| m.as_str())
+            .unwrap_or_else(|| panic!("no group at index '{}'", i))
+    }
+}
+
+/// Copied from [`regex::Captures`]...
+///
+/// Get a group by name.
+///
+/// `'t` is the lifetime of the matched text and `'i` is the lifetime
+/// of the group name (the index).
+///
+/// The text can't outlive the `Captures` object if this method is
+/// used, because of how `Index` is defined (normally `a[i]` is part
+/// of `a` and can't outlive it); to do that, use `name` instead.
+///
+/// # Panics
+///
+/// If there is no group named by the given value.
+impl<'t, 'i> Index<&'i str> for Captures<'t> {
+    type Output = str;
+
+    fn index<'a>(&'a self, name: &'i str) -> &'a str {
+        self.name(name)
+            .map(|m| m.as_str())
+            .unwrap_or_else(|| panic!("no group named '{}'", name))
+    }
+}
+
 impl<'c, 't> Iterator for SubCaptureMatches<'c, 't> {
     type Item = Option<Match<'t>>;
 
@@ -793,17 +1232,38 @@ fn push_usize(s: &mut String, x: usize) {
     }
 }
 
+fn is_special(c: char) -> bool {
+    match c {
+        '\\' | '.' | '+' | '*' | '?' | '(' | ')' | '|' | '[' | ']' | '{' | '}' | '^' | '$'
+        | '#' => true,
+        _ => false,
+    }
+}
+
 fn push_quoted(buf: &mut String, s: &str) {
     for c in s.chars() {
-        match c {
-            '\\' | '.' | '+' | '*' | '?' | '(' | ')' | '|' | '[' | ']' | '{' | '}' | '^' | '$'
-            | '#' => buf.push('\\'),
-            _ => (),
+        if is_special(c) {
+            buf.push('\\');
         }
         buf.push(c);
     }
 }
 
+/// Escapes special characters in `text` with '\\'.  Returns a string which, when interpreted
+/// as a regex, matches exactly `text`.
+pub fn escape(text: &str) -> Cow<str> {
+    // Using bytes() is OK because all special characters are single bytes.
+    match text.bytes().filter(|&b| is_special(b as char)).count() {
+        0 => Cow::Borrowed(text),
+        n => {
+            // The capacity calculation is exact because '\\' is a single byte.
+            let mut buf = String::with_capacity(text.len() + n);
+            push_quoted(&mut buf, text);
+            Cow::Owned(buf)
+        }
+    }
+}
+
 impl Expr {
     /// Parse the regex and return an expression (AST) and a bit set with the indexes of groups
     /// that are referenced by backrefs.
@@ -950,6 +1410,17 @@ fn codepoint_len(b: u8) -> usize {
     }
 }
 
+/// Returns the smallest possible index of the next valid UTF-8 sequence
+/// starting after `i`.
+/// Adapted from a function with the same name in the `regex` crate.
+fn next_utf8(text: &str, i: usize) -> usize {
+    let b = match text.as_bytes().get(i) {
+        None => return i + 1,
+        Some(&b) => b,
+    };
+    i + codepoint_len(b)
+}
+
 // If this returns false, then there is no possible backref in the re
 
 // Both potential implementations are turned off, because we currently
@@ -995,6 +1466,7 @@ mod tests {
     use crate::parse::make_literal;
     use crate::Expr;
     use crate::Regex;
+    use std::borrow::Cow;
     use std::usize;
     //use detect_possible_backref;
 
@@ -1043,6 +1515,20 @@ mod tests {
         assert_eq!(s, format!("{:?}", regex));
     }
 
+    #[test]
+    fn display() {
+        let s = r"(a+)b\1";
+        let regex = Regex::new(s).unwrap();
+        assert_eq!(s, format!("{}", regex));
+    }
+
+    #[test]
+    fn from_str() {
+        let s = r"(a+)b\1";
+        let regex = s.parse::<Regex>().unwrap();
+        assert_eq!(regex.as_str(), s);
+    }
+
     #[test]
     fn to_str_repeat() {
         fn repeat(lo: usize, hi: usize, greedy: bool) -> Expr {
@@ -1068,6 +1554,22 @@ mod tests {
         assert_eq!(to_str(repeat(1, usize::MAX, false)), "a+?");
     }
 
+    #[test]
+    fn escape() {
+        // Check that strings that need no quoting are borrowed, and that non-special punctuation
+        // is not quoted.
+        match crate::escape("@foo") {
+            Cow::Borrowed(s) => assert_eq!(s, "@foo"),
+            _ => panic!("Value should be borrowed."),
+        }
+
+        // Check typical usage.
+        assert_eq!(crate::escape("fo*o").into_owned(), "fo\\*o");
+
+        // Check that multibyte characters are handled correctly.
+        assert_eq!(crate::escape("fø*ø").into_owned(), "fø\\*ø");
+    }
+
     /*
     #[test]
     fn detect_backref() {
