@@ -149,6 +149,9 @@ impl<'a> Parser<'a> {
                 }
                 _ => return Ok((ix, child)),
             };
+            if !self.is_repeatable(&child) {
+                return Err(Error::TargetNotRepeatable);
+            }
             ix += 1;
             ix = self.optional_whitespace(ix)?;
             let mut greedy = true;
@@ -172,6 +175,18 @@ impl<'a> Parser<'a> {
         Ok((ix, child))
     }
 
+    fn is_repeatable(&self, child: &Expr) -> bool {
+        match child {
+            Expr::LookAround(_, _) => false,
+            Expr::Empty => false,
+            Expr::StartText => false,
+            Expr::EndText => false,
+            Expr::StartLine => false,
+            Expr::EndLine => false,
+            _ => true,
+        }
+    }
+
     // ix, lo, hi
     fn parse_repeat(&self, ix: usize) -> Result<(usize, usize, usize)> {
         let ix = self.optional_whitespace(ix + 1)?; // skip opening '{'
@@ -390,6 +405,10 @@ impl<'a> Parser<'a> {
 
     // ix points after '\x', eg to 'A0' or '{12345}', or after `\u` or `\U`
     fn parse_hex(&self, ix: usize, digits: usize) -> Result<(usize, Expr)> {
+        if ix >= self.re.len() {
+            // Incomplete escape sequence
+            return Err(Error::InvalidHex);
+        }
         let bytes = self.re.as_bytes();
         let b = bytes[ix];
         let (end, s) = if ix + digits <= self.re.len()
@@ -1312,6 +1331,19 @@ mod tests {
         assert_error("(?\u{1F60A})", "Unknown group flag: (?\u{1F60A}");
     }
 
+    #[test]
+    fn no_quantifiers_on_lookarounds() {
+        assert_error("(?=hello)+", "Target of repeat operator is invalid");
+        assert_error("(?<!hello)*", "Target of repeat operator is invalid");
+        assert_error("(?<=hello){2,3}", "Target of repeat operator is invalid");
+        assert_error("(?!hello)?", "Target of repeat operator is invalid");
+        assert_error("^?", "Target of repeat operator is invalid");
+        assert_error("${2}", "Target of repeat operator is invalid");
+        assert_error("(?m)^?", "Target of repeat operator is invalid");
+        assert_error("(?m)${2}", "Target of repeat operator is invalid");
+        assert_error("(a|b|?)", "Target of repeat operator is invalid");
+    }
+
     // found by cargo fuzz, then minimized
     #[test]
     fn fuzz_1() {
