@@ -84,7 +84,7 @@ const OPTION_TRACE: u32 = 1;
 const MAX_STACK: usize = 1_000_000;
 
 /// Instruction of the VM.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum Insn {
     /// Successful end of program
     End,
@@ -184,7 +184,7 @@ pub enum Insn {
 }
 
 /// Sequence of instructions for the VM to execute.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Prog {
     /// Instructions of the program
     pub body: Vec<Insn>,
@@ -409,7 +409,7 @@ pub fn run_default(prog: &Prog, s: &str, pos: usize) -> Result<Option<Vec<usize>
 }
 
 /// Run the program with options.
-#[allow(clippy::cyclomatic_complexity)]
+#[allow(clippy::cognitive_complexity)]
 pub(crate) fn run(
     prog: &Prog,
     s: &str,
@@ -672,7 +672,6 @@ pub(crate) fn run(
 mod tests {
     use super::*;
     use quickcheck::{quickcheck, Arbitrary, Gen};
-    use rand::Rng;
 
     #[test]
     fn state_push_pop() {
@@ -787,11 +786,14 @@ mod tests {
     }
 
     impl Arbitrary for Operation {
-        fn arbitrary<G: Gen>(g: &mut G) -> Self {
-            match g.gen_range(0, 3) {
-                0 => Operation::Push,
-                1 => Operation::Pop,
-                _ => Operation::Save(g.gen_range(0, 5), g.gen_range(0, usize::MAX)),
+        fn arbitrary(g: &mut Gen) -> Self {
+            match g.choose(&[0, 1, 2]) {
+                Some(0) => Operation::Push,
+                Some(1) => Operation::Pop,
+                _ => Operation::Save(
+                    *g.choose(&[0usize, 1, 2, 3, 4]).unwrap(),
+                    usize::arbitrary(g),
+                ),
             }
         }
     }
